class CalendarHelper {
    /**
     * 获取当前时间 （2018-08-15）
     */
    static getCurrentFormatDateString() {
        const date = new Date();
        return date.getFullYear() + "-" + CalenderUtils.getFullNumber(date.getMonth() + 1) + "-" + CalenderUtils.getFullNumber(date.getDate());
    }

    /**
     * 获取日历数据
     * @param year
     * @param month
     */
    static getCalendarData(year, month) {
        let newDatas = [];
        const monthInfo = new MonthInfo(year, month);
        const weekDay = CalenderUtils.getWeekDay(year, month);
        if (weekDay > 0) {
            //月份的第一天不是表的第一天。需要在前面插入上个月数据
            const lastMonthInfo = CalenderUtils.getLastMonthInfo(year, month);
            const lastMonthDays = CalenderUtils.getMonthDays(lastMonthInfo.year, lastMonthInfo.month)
            for (let j = 0; j < weekDay; j++) {
                const currentDay = lastMonthDays - weekDay + 1 + j;
                const dayInfo = new DayInfo(lastMonthInfo.year, lastMonthInfo.month, currentDay);
                dayInfo.setSupplementDay(true);
                newDatas.push(dayInfo);
            }
        }
        newDatas = newDatas.concat(monthInfo.dayItems);
        const size = newDatas.length;
        //为了列表显示（个数必须是7的倍数），必须补充空数据
        const emptyDays = 7 - size % 7;
        if (emptyDays < 7) {
            const nextMonthInfo = CalenderUtils.getNextMonthInfo(year, month)
            for (let j = 0; j < emptyDays; j++) {
                let dayInfo = new DayInfo(nextMonthInfo.year, nextMonthInfo.month,(j+1));
                dayInfo.setSupplementDay(true);
                newDatas.push(dayInfo)
            }
        }
        monthInfo.dayItems = newDatas;
        return monthInfo;
    }

    /**
     * 获取上个月的数据
     * @param year
     * @param month
     */
    static getLastMonthData(year, month) {
        const info = CalenderUtils.getLastMonthInfo(year, month);
        return CalendarHelper.getCalendarData(info.year, info.month)
    }

    /**
     * 获取下个月的数据
     * @param year
     * @param month
     */
    static getNextMonthData(year, month) {
        const info = CalenderUtils.getNextMonthInfo(year, month);
        return CalendarHelper.getCalendarData(info.year, info.month)
    }

    /**
     * 获取当前月份的日历数据
     */
    static getCurrentMonthData() {
        const date = new Date();
        return CalendarHelper.getCalendarData(date.getFullYear(), date.getMonth() + 1)
    }
    /**
     * 获取日历基础数据(从当前月，往后推)
     * @param count
     * @returns {Array}
     */
   static getCalendarDataList (count) {
        const months = [];
        let data = CalendarHelper.getCurrentMonthData();
        months.push(data)
        for (let i = 1; i < count; i++) {
            data = CalendarHelper.getNextMonthData(data.year, data.month);
            months.push(data)
        }
        return months;
    }
}

class MonthInfo {
    constructor(year, month) {
        this.titleName = month;
        this.year = year;
        this.month = month;
        this.dayItems = this.getDayItems();
    }

    getDayItems() {
        const days = CalenderUtils.getMonthDays(this.year, this.month);
        const dayItems = [];
        for (let i = 1; i <= days; i++) {
            const dayInfo = new DayInfo(this.year, this.month, i);
            dayItems.push(dayInfo)
        }
        return dayItems;
    }

    toString() {
        return "year= " + this.year + "  month= " + this.month;
    }
}

class DayInfo {
    constructor(year, month, day) {
        this.year = year;
        this.month = month;
        this.day = day;
        this.isSupplementDay = false;
        this.titleName = day;
        this.formatDay = this.getFormatDayString();
        const date = new Date(this.formatDay);
        date.setHours(0,0,0,0)
        this.time =date .getTime();
    }


    setSupplementDay(isSupplement) {
        this.isSupplementDay = isSupplement;
    }

    getFormatDayString() {
        const monthNumberString = CalenderUtils.getFullNumber(this.month);
        const dayNumberString = CalenderUtils.getFullNumber(this.day);
        return this.year + "-" + monthNumberString + "-" + dayNumberString;
    }

    equal(object) {
        if (object && object instanceof DayInfo) {
            return this.year === object.currentYear && this.month === object.currentMonth && this.day === object.currentDay
        }
        return false;
    }

    toString() {
        return "currentYear= " + this.year + "  currentMonth= " + this.month + "  currentDay= " + this.day;
    }
}

class CalenderUtils {

    static getFullNumber(num) {
        if (num >= 10) {
            return num;
        }
        return "0" + num;
    }

    static isLeapYear(year) {
        return year % 400 === 0 || (year % 4 === 0 && year % 100 !== 0);
    }

    /**
     * 获取指定月份天数
     * @param year
     * @param month
     * @returns {number}
     */
    static getMonthDays(year, month) {
        if (month == 2) {
            return CalenderUtils.isLeapYear(year) ? 29 : 28;
        } else if (month == 1 || month == 3 || month == 5 || month == 7 || month == 8 || month == 10 || month == 12) {
            return 31;
        }
        return 30;
    }

    static getNextMonthInfo(year, month) {
        const newMonth = month == 12 ? 1 : month + 1;
        const newYear = month == 12 ? year + 1 : year;
        return {year: newYear, month: newMonth}
    }

    static getLastMonthInfo(year, month) {
        //获取上个月实际月份
        const newMonth = month == 1 ? 12 : month - 1;
        //获取实际年份
        const newYear = month == 1 ? year - 1 : year;
        return {year: newYear, month: newMonth}
    }

    /**
     * 获取某年某月第一天是星期几
     * @param year
     * @param month
     */
    static getWeekDay(year, month) {
        const date = new Date(year, month - 1, 1);
        return date.getDay();
    }
}

module.exports = CalendarHelper
