const dateConstants = require('./DateConstants')
class CountDownTimer {
    /**
     *
     * @param millisInFuture 时间毫秒数
     * @param countDownInterval 时间毫秒数（单位间隔）
     */
    constructor(millisInFuture, countDownInterval) {
        this.millisInFuture = millisInFuture;
        this.mInterval = countDownInterval;
        this.onFinishListener =undefined;
        this.onCountDownListener=undefined;
        this.timer = undefined;
    }

   addCountDownListener(listener){
       if(listener && typeof listener ==='function'){
           this.onCountDownListener=listener;
       }
   }

   addFinishListener(listener){
        if(listener && typeof listener ==='function'){
            this.onFinishListener=listener;
        }
   }

    start() {
        if (!this.timer && this.onCountDownListener && typeof this.onCountDownListener==='function') {
            let that = this;
            this.timer = setInterval(() => {
                that.millisInFuture = that.millisInFuture - that.mInterval;
                if (that.millisInFuture > 0) {
                    that.onCountDownListener(CountDownTimer.getTimeInfo(that.millisInFuture));
                } else {
                    that.cancel();
                    if(that.onFinishListener && typeof that.onFinishListener ==='function'){
                        that.onFinishListener();
                    }
                }
            }, that.mInterval);
        }
    }

    cancel() {
        if (this.timer) {
            clearInterval(this.timer);
            this.timer = undefined;
        }
    }

    static getTimeInfo(time) {
        let hourTime = time % dateConstants.dayMillis;//剩余的小时部分时间
        let minuteTime = time % dateConstants.hourMillis;//剩余的分钟部分时间
        let secondTime = time % dateConstants.minuteMillis;//剩余的秒部分时间
        return {
            time: time,
            day:Math.floor(time/dateConstants.dayMillis),
            hour: Math.floor(hourTime /dateConstants.hourMillis),
            minute: Math.floor(minuteTime /dateConstants.minuteMillis),
            second: Math.floor(secondTime /dateConstants.secondMillis),
            format: (value) => {
                return value >= 10 ? value : "0" + value;
            }
        }
    }
}

module.exports = CountDownTimer;
