//克隆对象
function cloneObject(object) {
	if (object instanceof Array) {
		let newArray = [];
		for (let i = 0, len = object.length; i < len; i++) {
			newArray.push(cloneObject(object[i]));
		}
		return newArray;
	} else if (object instanceof Object) {
		let newObject = {};
		for (let key in object) {
			newObject[key] = cloneObject(object[key])
		}
		return newObject;
	}
	return object;
}

//将某个对象追加到当前对象中(扩展对象，类似java中的继承)
function extendObject(targetObject, sourceObject) {
	if (!sourceObject || typeof (sourceObject) !== 'object') {//源对象为空或者不是对象时
		return targetObject;
	}
	if (targetObject && typeof (sourceObject) === 'object') {
		Object.keys(sourceObject).forEach(sourceKey=>{
			if (typeof (targetObject[sourceKey]) === 'undefined') {//当前目标没有定义
				targetObject[sourceKey] = sourceObject[sourceKey];
			} else if (typeof (targetObject[sourceKey]) === 'object') {//当前目标不为空而且是对象时
				extendObject(targetObject[sourceKey], sourceObject[sourceKey])
			}
		})
	}
	return targetObject;
}

//判断列表是是否包含特定的对象
function containObject(list, object) {
	if (!list || list.length < 1) {
		return false
	}
	for (let i = 0, len = list.length; i < len; i++) {
        if(deepEqual(list[i],object)){
          return true;
        }
	}
	return false;
}

//深入比较对象(判断对象是否相等)
function deepEqual(object_1, object_2) {
	let type1 = typeof object_1
	let type2 = typeof object_2
	if (type1 !== type2) {
		return false
	}
	if (type1 === 'string' || type1 === 'number' || type1 === 'undefined' || type1 === 'boolean') {
		return object_1 === object_2
	}
	if (type1 === 'object') {
		if (type1 instanceof Array) {
			if (object_1.length !== object_1.length) {
				return false
			}
			if (!object_1.every(item => containObject(object_2, item))) {
				return false
			}
			if (!object_2.every(item => containObject(object_1, item))) {
				return false
			}
			return true
		} else {
			let keys_1 = Object.keys(object_1)
			let keys_2 = Object.keys(object_2)
			if (keys_1.length !== keys_2.length) {
				return false
			}
			let filterKeys = keys_2.filter(key => keys_1.includes(key))
			if (filterKeys.length !== keys_1.length) {
				return false
			}
			for (let i = 0, len = keys_1.length; i < len; i++) {
				let equal = deepEqual(object_1[keys_1[i]], object_2[keys_1[i]])
				if (!equal) {
					return false
				}
			}
			return true
		}
	}
	return false;
}

function getObjectByList(keysList, valuesObject) {
	let newObject = {}
	if (keysList && keysList.length > 0 && valuesObject && typeof valuesObject === 'object') {
		for (let i = 0, len = keysList.length; i < len; i++) {
			let key = keysList[i];
			if (key) {
				newObject[key] = valuesObject[key]
			}
		}
	}
	return newObject
}

function getObjectByObject(keysObject, targetObject) {
	if (keysObject && targetObject && typeof keysObject === 'object' && typeof targetObject === 'object') {
		let keys = Object.keys(keysObject)
		return getObjectByList(keys,targetObject);
	}
	return {};
}
function getValueFromObject(obj,keyWords){
	if(typeof  obj ==='object' && keyWords && typeof keyWords==='string'){
		if(obj[keyWords]){
			return obj[keyWords];
		}
		let indexOf = keyWords.indexOf(".");
		if(indexOf<0){
			return obj[keyWords]
		}else{
			const keys=keyWords.split(".")
			if(obj[keys[0]] &&  obj[keys[0]] instanceof Object){
				const nextKeyWorks = keys.slice(1).join('.')
				return getValueFromObject(obj[keys[0]] ,nextKeyWorks);
			}
		}
	}
	return "";
}
/**
 * 监听属性 并执行监听函数
 */
function observe(obj, key,watchFun) {
	if(obj && typeof obj==='object'){
		let val = obj[key]; //给该属性设默认值
		if(val){
			Object.defineProperty(obj, key, {
				configurable: true,
				enumerable: true,
				set: function(value) {
					let oldVal = val;
					val = value;
					watchFun(value, oldVal);
				},
				get: function() {
					return val;
				}
			})
		}else{
			let indexOf = key.indexOf(".");
			if(indexOf>0){
				const objKey=key.substring(0,indexOf)
				let newKey=key.substr(indexOf+1);
				observe(obj[objKey],newKey,watchFun)
			}
		}
	}
}
/**
 * 设置监听器
 */
function setWatcher(data, watch) {
	Object.keys(watch).forEach(v => {
		observe(data, v,watch[v]);
	})
}
module.exports = {
	getObjectByObject:getObjectByObject,
	getObjectByList:getObjectByList,
	getValueFromObject:getValueFromObject,
	deepEqual:deepEqual,
	containObject:containObject,
	cloneObject:cloneObject,
	extendObject:extendObject,
	setWatcher:setWatcher,
};
