const IdCardHelper = require("./IdCardHelper")
class StringHelper {
  /**
   * 替换所有
   * @param content 当前文本
   * @param pattern 旧字符串或者正则表达式
   * @param newStr 要替换的内容
   * @returns {*}  返回新的字符串
   */
  static replaceAll(content, pattern, newStr) {
    const reg = new RegExp(pattern, "g"); //创建正则RegExp对象
    return content.replace(reg, newStr)
  }

  /**
   * 判断是否是身份证
   * @param codeStr
   * @returns {boolean}
   */
  static isIDCard(codeStr){
    return IdCardHelper.checkCard(codeStr)
  }
  static isEmpty(str){
     return !str || str.trim().length < 1;
  }
  /**
   * 校验手机号（严格校验）
   * //移动号段：134 135 136 137 138 139 147 148 150 151 152 157 158 159  165 172 178 182 183 184 187 188 195 198
   * // 联通号段：130 131 132 145 146 155 156 166 167 171 175 176 185 186
   * // 电信号段：133 149 153 173 174 177 180 181 189  191  199
   * // 虚拟运营商:170
   * 分组如下
   * //130 131 132 133 134 135 136 137 138 139
   * //145 146 147 148 149
   * //150 151 152 153 155 156 157 158 159
   * // 165 166 167
   * //170  171  172 173 174  175 176 177 178
   * //180 181 182 183 184 185 186 187 188 189
   * // 191  195  198 199
   * @param phone
   * @returns {boolean}
   */
  static isPhoneNumber(phone) {
    const phoneStr=String(phone);
    //手机号正则
    if (/^1[38][0-9]{9}$/.test(phoneStr)) {//校验//130 131 132 133 134 135 136 137 138 139  180 181 182 183 184 185 186 187 188 189号段
      return true;
    }
    if (/^14[56789][0-9]{8}$/.test(phoneStr)) {//校验14开头的号段
      return true
    }
    if (/^15[012356789][0-9]{8}$/.test(phoneStr)) {//校验 1//150 151 152 153 155 156 157 158 159 开头的号段
      return true;
    }
    if (/^16[567][0-9]{8}$/.test(phoneStr)) {//校验 // 165 166 167 开头的号段
      return true;
    }
    if (/^17[012345678][0-9]{8}$/.test(phoneStr)) {//校验 //170  171  172 173 174  175 176 177 178 开头的号段
      return true;
    }
    if (/^19[1589][0-9]{8}$/.test(phoneStr)) {//校验// 191  195  198 199 开头的号段
      return true;
    }
    return false;
  }

  /**
   * 判断是否是邮件地址
   * @param email
   * @returns {boolean}
   */
  static isEmail(email) {
    if (email) {
      const regex = /^[a-zA-Z0-9]+([-_.][a-zA-Z0-9]+)*@[a-zA-Z0-9]+([-_.][a-zA-Z0-9]+)*\.[a-z]{2,}$/
      return regex.test(email)
    }
    return false
  }
  /**
   * 从富文本中，查找img 标签，收集图片地址(只支持.png .jpg .jpeg 结尾)
   * @param text 富文本
   * @returns {[]}  返回图片列表
   */
  static getImageListFromRichText(text){
    const list = [];
    let regular = /<img[^>]*>/g
    let keysWords = text.match(regular)
    if(keysWords && keysWords.length>0){
      let srcReg = /http\S*((.jpg)|(.jpeg)|(.png))/g
      keysWords.forEach(words=>{
        let strings= words.match(srcReg);
        if(strings && strings.length>0 && !list.includes(strings[0])){
          list.push(strings[0])
        }
      })
    }
    return list ;
  }
  /*RGB颜色转换为16进制*/
  static colorRbgToHex(rgbColor){
    let lowerCaseColor = rgbColor.toLowerCase();
    let reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
    if (/^(rgb|RGB)/.test(lowerCaseColor)) {
      let aColor = lowerCaseColor.replace(/(?:\(|\)|rgb|RGB)*/g, "").split(",");
      let strHex = "#";
      for (let i=0; i<aColor.length; i++) {
        let hex = Number(aColor[i]).toString(16);
        if (hex === "0") {
          hex += hex;
        }
        strHex += hex;
      }
      if (strHex.length !== 7) {
        strHex = lowerCaseColor;
      }
      return strHex;
    } else if (reg.test(lowerCaseColor)) {
      let aNum = lowerCaseColor.replace(/#/,"").split("");
      if (aNum.length === 6) {
        return lowerCaseColor;
      } else if(aNum.length === 3) {
        let numHex = "#";
        for (let i=0; i<aNum.length; i+=1) {
          numHex += (aNum[i] + aNum[i]);
        }
        return numHex;
      }
    }
    return lowerCaseColor;
  }

  /**
   *16进制颜色转为RGB格式
   * @param hexColor  16进制颜色
   * @returns {string}  RGB格式颜色
   */
  static colorHexToRgb(hexColor){
    let sColor = hexColor.toLowerCase();
    let reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
    if (sColor && reg.test(sColor)) {
      if (sColor.length === 4) {
        let sColorNew = "#";
        for (let i=1; i<4; i+=1) {
          sColorNew += sColor.slice(i, i+1).concat(sColor.slice(i, i+1));
        }
        sColor = sColorNew;
      }
      //处理六位的颜色值
      let sColorChange = [];
      for (let i=1; i<7; i+=2) {
        sColorChange.push(parseInt("0x"+sColor.slice(i, i+2)));
      }
      return "rgb(" + sColorChange.join(",") + ")";
    }
    return sColor;
  };

  /**
   * 将整数部分逢三一断
   * @param value
   * @returns {string}
   */
  static formatNumber(value) {
    if (!value) {
      return '0'
    }
    return value.toString().replace(/(\d)(?=(?:\d{3})+$)/g, '$1,')
  }

}

module.exports = StringHelper;
